{
  "type": "record",
  "Aspect": {
    "name": "assertionInferenceAdjustmentRuleInfo",
    "schemaVersion": 1
  },
  "name": "AssertionInferenceAdjustmentRuleInfo",
  "namespace": "com.linkedin.pegasus2avro.assertion.adjustment",
  "fields": [
    {
      "Searchable": {
        "fieldType": "TEXT_PARTIAL"
      },
      "type": "string",
      "name": "name",
      "doc": "Display name of the rule."
    },
    {
      "Searchable": {
        "fieldType": "KEYWORD"
      },
      "type": {
        "type": "enum",
        "symbolDocs": {
          "DISABLED": "The rule is paused and ignored by training and evaluation.",
          "ENABLED": "The rule is actively applied during assertion-inference training and evaluation."
        },
        "name": "AssertionInferenceAdjustmentRuleMode",
        "namespace": "com.linkedin.pegasus2avro.assertion.adjustment",
        "symbols": [
          "ENABLED",
          "DISABLED"
        ],
        "doc": "The mode of an Assertion Inference Adjustment Rule."
      },
      "name": "mode",
      "default": "ENABLED",
      "doc": "The mode of the rule (ENABLED or DISABLED)."
    },
    {
      "type": {
        "type": "record",
        "name": "AssertionInferenceAdjustmentRuleFilter",
        "namespace": "com.linkedin.pegasus2avro.assertion.adjustment",
        "fields": [
          {
            "type": {
              "type": "record",
              "name": "Filter",
              "namespace": "com.linkedin.pegasus2avro.metadata.query.filter",
              "fields": [
                {
                  "type": [
                    "null",
                    {
                      "type": "array",
                      "items": {
                        "type": "record",
                        "name": "ConjunctiveCriterion",
                        "namespace": "com.linkedin.pegasus2avro.metadata.query.filter",
                        "fields": [
                          {
                            "type": {
                              "type": "array",
                              "items": {
                                "type": "record",
                                "name": "Criterion",
                                "namespace": "com.linkedin.pegasus2avro.metadata.query.filter",
                                "fields": [
                                  {
                                    "type": "string",
                                    "name": "field",
                                    "doc": "The name of the field that the criterion refers to"
                                  },
                                  {
                                    "type": {
                                      "type": "array",
                                      "items": "string"
                                    },
                                    "name": "values",
                                    "default": [],
                                    "doc": "Values to match against the field (one match suffices unless otherwise documented)."
                                  },
                                  {
                                    "type": {
                                      "type": "enum",
                                      "symbolDocs": {
                                        "ANCESTORS_INCL": "Represent the relation: URN field matches any nested parent in addition to the given URN",
                                        "BETWEEN": "Represent the relation within an inclusive range, e.g. 3 <= ownerCount <= 5. Note that the values in Criterion must have two entries for a lower and upper bound.",
                                        "CONTAIN": "Represent the relation: String field contains value, e.g. name contains Profile",
                                        "DESCENDANTS_INCL": "Represent the relation: URN field any nested children in addition to the given URN",
                                        "END_WITH": "Represent the relation: String field ends with value, e.g. name ends with Event",
                                        "EQUAL": "Represent the relation: field = value, e.g. platform = hdfs",
                                        "EXISTS": "Represents the relation: field exists and is non-empty, e.g. owners is not null and != [] (empty)",
                                        "GREATER_THAN": "Represent the relation greater than, e.g. ownerCount > 5",
                                        "GREATER_THAN_OR_EQUAL_TO": "Represent the relation greater than or equal to, e.g. ownerCount >= 5",
                                        "IEQUAL": "Represent the relation: field = value and support case insensitive values, e.g. platform = hdfs",
                                        "IN": "Represent the relation: String field is one of the array values to, e.g. name in [\"Profile\", \"Event\"]",
                                        "IS_NULL": "Represent the relation: field is null, e.g. platform is null",
                                        "LESS_THAN": "Represent the relation less than, e.g. ownerCount < 3",
                                        "LESS_THAN_OR_EQUAL_TO": "Represent the relation less than or equal to, e.g. ownerCount <= 3",
                                        "RELATED_INCL": "Represent the relation: URN field matches any nested child or parent in addition to the given URN",
                                        "START_WITH": "Represent the relation: String field starts with value, e.g. name starts with PageView"
                                      },
                                      "name": "Condition",
                                      "namespace": "com.linkedin.pegasus2avro.metadata.query.filter",
                                      "symbols": [
                                        "CONTAIN",
                                        "END_WITH",
                                        "EQUAL",
                                        "IEQUAL",
                                        "IS_NULL",
                                        "EXISTS",
                                        "GREATER_THAN",
                                        "GREATER_THAN_OR_EQUAL_TO",
                                        "IN",
                                        "LESS_THAN",
                                        "LESS_THAN_OR_EQUAL_TO",
                                        "BETWEEN",
                                        "START_WITH",
                                        "DESCENDANTS_INCL",
                                        "ANCESTORS_INCL",
                                        "RELATED_INCL"
                                      ],
                                      "doc": "The matching condition in a filter criterion"
                                    },
                                    "name": "condition",
                                    "default": "EQUAL",
                                    "doc": "The condition for the criterion, e.g. EQUAL, START_WITH"
                                  },
                                  {
                                    "type": "boolean",
                                    "name": "negated",
                                    "default": false,
                                    "doc": "Whether the condition should be negated"
                                  }
                                ],
                                "doc": "A criterion for matching a field with given values"
                              }
                            },
                            "name": "and",
                            "doc": "A list of and criteria the filter applies to the query"
                          }
                        ],
                        "doc": "A list of criterion and'd together."
                      }
                    }
                  ],
                  "name": "or",
                  "default": null,
                  "doc": "A list of disjunctive criterion for the filter. (or operation to combine filters)"
                },
                {
                  "type": [
                    "null",
                    {
                      "type": "array",
                      "items": "com.linkedin.pegasus2avro.metadata.query.filter.Criterion"
                    }
                  ],
                  "name": "criteria",
                  "default": null,
                  "doc": "Deprecated! A list of conjunctive criterion for the filter. If \"or\" field is provided, then this field is ignored."
                }
              ],
              "doc": "The filter for finding a record or a collection of records"
            },
            "name": "filter",
            "doc": "The structured filter used to match entities the adjustment rule applies to."
          },
          {
            "type": [
              "null",
              "string"
            ],
            "name": "json",
            "default": null,
            "doc": "The serialized JSON representation of the filter, used for UI rendering.\nThe UI builds a logical predicate tree that is flattened into orFilters for GraphQL.\nThis string preserves the original tree structure so the UI can restore it."
          }
        ],
        "doc": "The filter criteria used to match entities for an Assertion Inference Adjustment Rule."
      },
      "name": "entityFilter",
      "doc": "The filter criteria used to match entities this rule applies to."
    },
    {
      "type": [
        "null",
        {
          "type": "record",
          "name": "AssertionInferenceExclusionConfig",
          "namespace": "com.linkedin.pegasus2avro.assertion.adjustment",
          "fields": [
            {
              "Searchable": {
                "/*/fixedRange/endTimeMillis": {
                  "fieldName": "assertionInferenceFixedExclusionWindowEndTimes",
                  "fieldType": "DATETIME",
                  "queryByDefault": false
                },
                "/*/fixedRange/startTimeMillis": {
                  "fieldName": "assertionInferenceFixedExclusionWindowStartTimes",
                  "fieldType": "DATETIME",
                  "queryByDefault": false
                },
                "/*/holiday/name": {
                  "fieldName": "assertionInferenceHolidayExclusionWindowNames",
                  "fieldType": "KEYWORD",
                  "queryByDefault": false
                },
                "/*/holiday/region": {
                  "fieldName": "assertionInferenceHolidayExclusionWindowRegions",
                  "fieldType": "KEYWORD",
                  "queryByDefault": false
                },
                "/*/holiday/timezone": {
                  "fieldName": "assertionInferenceHolidayExclusionWindowTimezones",
                  "fieldType": "KEYWORD",
                  "queryByDefault": false
                },
                "/*/type": {
                  "fieldName": "assertionInferenceExclusionWindowTypes",
                  "fieldType": "KEYWORD",
                  "queryByDefault": false
                },
                "/*/weekly/daysOfWeek/*": {
                  "fieldName": "assertionInferenceWeeklyExclusionWindowDaysOfWeek",
                  "fieldType": "KEYWORD",
                  "queryByDefault": false
                },
                "/*/weekly/endTime": {
                  "fieldName": "assertionInferenceWeeklyExclusionWindowEndTimes",
                  "fieldType": "KEYWORD",
                  "queryByDefault": false
                },
                "/*/weekly/startTime": {
                  "fieldName": "assertionInferenceWeeklyExclusionWindowStartTimes",
                  "fieldType": "KEYWORD",
                  "queryByDefault": false
                },
                "/*/weekly/timezone": {
                  "fieldName": "assertionInferenceWeeklyExclusionWindowTimezones",
                  "fieldType": "KEYWORD",
                  "queryByDefault": false
                }
              },
              "type": {
                "type": "array",
                "items": {
                  "type": "record",
                  "name": "AssertionExclusionWindow",
                  "namespace": "com.linkedin.pegasus2avro.assertion",
                  "fields": [
                    {
                      "type": {
                        "type": "enum",
                        "symbolDocs": {
                          "FIXED_RANGE": "One-off time range to exclude",
                          "HOLIDAY": "Specific holiday to exclude",
                          "WEEKLY": "Recurring time range to exclude each week"
                        },
                        "name": "AssertionExclusionWindowType",
                        "namespace": "com.linkedin.pegasus2avro.assertion",
                        "symbols": [
                          "FIXED_RANGE",
                          "WEEKLY",
                          "HOLIDAY"
                        ]
                      },
                      "name": "type",
                      "doc": "The type of exclusion window."
                    },
                    {
                      "type": [
                        "null",
                        "string"
                      ],
                      "name": "displayName",
                      "default": null,
                      "doc": "Display name for this exclusion window"
                    },
                    {
                      "type": [
                        "null",
                        {
                          "type": "record",
                          "name": "AbsoluteTimeWindow",
                          "namespace": "com.linkedin.pegasus2avro.timeseries",
                          "fields": [
                            {
                              "type": "long",
                              "name": "startTimeMillis",
                              "doc": "Start time as epoch at UTC."
                            },
                            {
                              "type": "long",
                              "name": "endTimeMillis",
                              "doc": "End time as epoch at UTC."
                            }
                          ]
                        }
                      ],
                      "name": "fixedRange",
                      "default": null,
                      "doc": "Date window for one-off exclusion range\nOnly used when type is FIXED_RANGE"
                    },
                    {
                      "type": [
                        "null",
                        {
                          "type": "record",
                          "name": "WeeklyWindow",
                          "namespace": "com.linkedin.pegasus2avro.timeseries",
                          "fields": [
                            {
                              "type": {
                                "type": "array",
                                "items": {
                                  "type": "enum",
                                  "name": "DayOfWeek",
                                  "namespace": "com.linkedin.pegasus2avro.timeseries",
                                  "symbols": [
                                    "MONDAY",
                                    "TUESDAY",
                                    "WEDNESDAY",
                                    "THURSDAY",
                                    "FRIDAY",
                                    "SATURDAY",
                                    "SUNDAY"
                                  ]
                                }
                              },
                              "name": "daysOfWeek",
                              "doc": "Days of the week to include in the window.\nIf not specified, all days are included."
                            },
                            {
                              "type": [
                                "null",
                                "string"
                              ],
                              "name": "startTime",
                              "default": null,
                              "doc": "Start time for the window on each specified day.\nFormat: \"HH:MM\" in 24-hour format (e.g., \"09:00\" for 9am, \"17:30\" for 5:30pm).\nIf not specified, defaults to start of day (\"00:00\")."
                            },
                            {
                              "type": [
                                "null",
                                "string"
                              ],
                              "name": "endTime",
                              "default": null,
                              "doc": "End time for the window on each specified day.\nFormat: \"HH:MM\" in 24-hour format (e.g., \"17:00\" for 5pm, \"23:59\" for end of day).\nIf not specified, defaults to end of day (\"23:59\")."
                            },
                            {
                              "type": [
                                "null",
                                "string"
                              ],
                              "name": "timezone",
                              "default": null,
                              "doc": "Time zone to interpret start and end times in.\nUses standard time zone identifiers (e.g., \"America/Los_Angeles\", \"UTC\").\nIf not specified, defaults to UTC."
                            }
                          ],
                          "doc": "Represents a recurring time window that repeats weekly.\nUsed to define exclusion periods for model training based on day of week and time of day."
                        }
                      ],
                      "name": "weekly",
                      "default": null,
                      "doc": "Window of time each week to exclude."
                    },
                    {
                      "type": [
                        "null",
                        {
                          "type": "record",
                          "name": "HolidayWindow",
                          "namespace": "com.linkedin.pegasus2avro.timeseries",
                          "fields": [
                            {
                              "type": "string",
                              "name": "name",
                              "doc": "The name of the holiday."
                            },
                            {
                              "type": [
                                "null",
                                "string"
                              ],
                              "name": "region",
                              "default": null,
                              "doc": "The region of the holiday.\nExample: 'US'"
                            },
                            {
                              "type": [
                                "null",
                                "string"
                              ],
                              "name": "timezone",
                              "default": null,
                              "doc": "The timezone of the holiday.\nExample: 'America/New_York'"
                            }
                          ]
                        }
                      ],
                      "name": "holiday",
                      "default": null,
                      "doc": "The holiday to exclude\nOnly used when type is HOLIDAY"
                    }
                  ],
                  "doc": "Information about an assertion exclusion window.\nThis is used to exclude specific time periods from assertion evaluation or training.\nFor example, excluding holidays, weekends, or known anomalous periods.\nSupports various recurrence patterns: one-off exclusions, or recurring exclusions\n(yearly, monthly, weekly, daily, or specific holidays)."
                }
              },
              "name": "exclusionWindows",
              "default": [],
              "doc": "The set of exclusion windows contributed by this rule."
            }
          ],
          "doc": "Configuration for exclusion windows applied by an Assertion Inference Adjustment Rule.\nWhen the rule's filter matches a dataset, these windows are unioned with any\nper-monitor exclusion windows at both training and evaluation time."
        }
      ],
      "name": "exclusionConfig",
      "default": null,
      "doc": "Optional exclusion-window configuration contributed by this rule.\nWhen set, its windows are unioned with per-monitor windows\nat training and evaluation time."
    },
    {
      "type": {
        "type": "record",
        "name": "AuditStamp",
        "namespace": "com.linkedin.pegasus2avro.common",
        "fields": [
          {
            "type": "long",
            "name": "time",
            "doc": "When did the resource/association/sub-resource move into the specific lifecycle stage represented by this AuditEvent."
          },
          {
            "java": {
              "class": "com.linkedin.pegasus2avro.common.urn.Urn"
            },
            "type": "string",
            "name": "actor",
            "doc": "The entity (e.g. a member URN) which will be credited for moving the resource/association/sub-resource into the specific lifecycle stage. It is also the one used to authorize the change.",
            "Urn": "Urn"
          },
          {
            "java": {
              "class": "com.linkedin.pegasus2avro.common.urn.Urn"
            },
            "type": [
              "null",
              "string"
            ],
            "name": "impersonator",
            "default": null,
            "doc": "The entity (e.g. a service URN) which performs the change on behalf of the Actor and must be authorized to act as the Actor.",
            "Urn": "Urn"
          },
          {
            "type": [
              "null",
              "string"
            ],
            "name": "message",
            "default": null,
            "doc": "Additional context around how DataHub was informed of the particular change. For example: was the change created by an automated process, or manually."
          }
        ],
        "doc": "Data captured on a resource/association/sub-resource level giving insight into when that resource/association/sub-resource moved into a particular lifecycle stage, and who acted to move it into that specific lifecycle stage."
      },
      "name": "created",
      "doc": "Audit stamp capturing when and by whom this rule was created."
    },
    {
      "type": [
        "null",
        "com.linkedin.pegasus2avro.common.AuditStamp"
      ],
      "name": "updated",
      "default": null,
      "doc": "Audit stamp capturing when and by whom this rule was last updated."
    }
  ],
  "doc": "Information about an Assertion Inference Adjustment Rule.\nAn Assertion Inference Adjustment Rule globally adjusts assertion-inference\ntraining and evaluation for entities matching a filter \u2014 for example, by contributing\nexclusion windows that cover known maintenance periods or holidays. Additional\nsibling configs (e.g. changepoints) may be added in the future."
}