{
  "type": "record",
  "Aspect": {
    "name": "assertionInferenceAdjustmentRuleKey",
    "keyForEntity": "assertionInferenceAdjustmentRule",
    "entityCategory": "core",
    "entityAspects": [
      "assertionInferenceAdjustmentRuleInfo",
      "ownership",
      "status"
    ],
    "entityDoc": "A globally-applied rule that adjusts assertion-inference training and evaluation for entities matching a filter (e.g. exclusion windows for known maintenance periods or holidays)."
  },
  "name": "AssertionInferenceAdjustmentRuleKey",
  "namespace": "com.linkedin.pegasus2avro.metadata.key",
  "fields": [
    {
      "type": "string",
      "name": "id",
      "doc": "Unique id for the assertion inference adjustment rule."
    }
  ],
  "doc": "Key for an Assertion Inference Adjustment Rule."
}